package bst;

import org.junit.BeforeClass;
import org.junit.Test;

import static org.junit.Assert.*;

public class BinarySearchTreeTest {

    static BinarySearchTree bstSearching;

    @BeforeClass
    public static void setup() {
        BSTNode<Integer> rootSearching = new BSTInternal<>(64,
                                                new BSTInternal<>(32, new BSTLeaf<>(16), new BSTLeaf<>(40)),
                                                new BSTInternal<>(80, null, new BSTLeaf<>(96)));
        bstSearching = new BinarySearchTree<Integer>(rootSearching);
    }

    @Test
    public void contains() {

        int[] members = {16, 32, 40, 64, 80, 96};
        int[] nonMembers = {-10, 20, 38, 41, 68, 81, 100};

        for (int member : members) {
            assertTrue(bstSearching.contains(member));
        }

        for (int nonMember : nonMembers) {
            assertFalse(bstSearching.contains(nonMember));
        }
    }

    @Test
    public void add() {
        BinarySearchTree<String> bstAdding = new BinarySearchTree<>();
        String[] insertions = {"20", "12", "23", "11", "30", "21" };
        for (String insertion : insertions) {
            assertTrue(bstAdding.add(insertion));
        }

        BSTInternal<String> root = (BSTInternal<String>) (bstAdding.root());
        BSTInternal<String> l = (BSTInternal<String>) root.getLeft();
        BSTNode<String> ll = l.getLeft();
        BSTNode<String> lr = l.getRight();
        BSTInternal<String> r = (BSTInternal<String>) root.getRight();
        BSTNode<String> rl = r.getLeft();
        BSTNode<String> rr = r.getRight();
        assertEquals( "20", root.getValue());
        assertEquals("12", l.getValue());
        assertEquals("11", ll.getValue());
        assertNull(lr);
        assertEquals("23", r.getValue());
        assertEquals("21", rl.getValue());
        assertEquals("30", rr.getValue());
    }

    @Test
    public void remove() {
        BinarySearchTree<Integer> removalTree = new BinarySearchTree<>();
        Integer[] insertions = {19, 14, 25, 21, 30, 27, 68 };
        for (int insertion : insertions) {
            assertTrue(removalTree.add(insertion));
        }

        removalTree.remove(25);
        BSTInternal<Integer> root = (BSTInternal<Integer>) (removalTree.root());
        BSTNode<Integer> l = root.getLeft();
        BSTInternal<Integer> r = (BSTInternal<Integer>) root.getRight();
        BSTNode<Integer> rl = r.getLeft();
        BSTInternal<Integer> rr = (BSTInternal<Integer>) r.getRight();
        BSTNode<Integer> rrr = rr.getRight();
        BSTNode<Integer> rrl = rr.getLeft();

        assertEquals(Integer.valueOf(19), root.getValue());
        assertEquals(Integer.valueOf(14), l.getValue());
        assertEquals(Integer.valueOf(27), r.getValue());
        assertEquals(Integer.valueOf(21), rl.getValue());
        assertEquals(Integer.valueOf(30), rr.getValue());
        assertEquals(Integer.valueOf(68), rrr.getValue());
        assertNull(rrl);
    }

    @Test
    public void treeMap() {

        TreeMap<String, Integer> tm = new TreeMap<>();

        tm.put("Harry", 4);
        tm.put("Daniel", 17);
        tm.put("Wenyi", -30);

        assertEquals(Integer.valueOf(4), tm.get("Harry"));
        assertEquals(Integer.valueOf(17), tm.get("Daniel"));
        assertEquals(Integer.valueOf(-30), tm.get("Wenyi"));

        tm.put("Harry", 40);
        assertEquals(Integer.valueOf(40), tm.get("Harry"));

    }
}