package bst;

import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

class SymbolTableTest {

    //Adding a new symbol to an empty table :white_check_mark:
    @Test
    public void addToEmptyTable() {
        SymbolTable symbolTable = new SymbolTable();
        assertTrue(symbolTable.add("x", "int"));
    }

    //Adding a new symbol to a non-empty table :white_check_mark:
    @Test
    public void addToNonEmptyTable() {
        SymbolTable symbolTable = new SymbolTable();
        symbolTable.add("x", "int");
        assertTrue(symbolTable.add("y", "int"));
    }

    //Adding a symbol that is already in the table (:x:, redeclaring a variable!)
    @Test
    public void addExistingSymbol() {
        SymbolTable symbolTable = new SymbolTable();
        symbolTable.add("x", "int");
        assertThrows(IllegalArgumentException.class, () -> symbolTable.add("x", "int"));
    }

    //Checking for a symbol that is present in the table :white_check_mark:
    @Test
    public void containsExistingSymbol() {
        SymbolTable symbolTable = new SymbolTable();
        symbolTable.add("x", "int");
        assertTrue(symbolTable.contains("x", "int"));
    }
    //Checking for a symbol that is not present in the table :white_check_mark:
    @Test
    public void containsNonExistingSymbol() {
        SymbolTable symbolTable = new SymbolTable();
        assertFalse(symbolTable.contains("x", "int"));
    }

    //Removing a symbol that is present in the table :white_check_mark:
    @Test
    public void removeExistingSymbol() {
        SymbolTable symbolTable = new SymbolTable();
        symbolTable.add("x", "int");
        assertTrue(symbolTable.remove("x", "int"));
        assertFalse(symbolTable.contains("x", "int"));
    }
    //Removing a symbol that is not present in the table (:x:, shouldn't happen)
    @Test
    public void removeNonexistentSymbol() {
        SymbolTable symbolTable = new SymbolTable();
        assertThrows(IllegalArgumentException.class, () -> symbolTable.remove("x", "int"));
    }

}